# the implemented measurements
MEASUREMENTS = (
    "tide",
    "wind",
    "water_temp",
    "air_temp",
    "air_press",
    "wave",
    "conductivity",
)

# the integrated APIs
# These are == to the filenames in api/ and /data
DATASOURCES = (
    "noaa_ndbc",
    "noaa_tidesandcurrents",
)
